//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  (C) 2019: Philipp Thomas König
//  E-Mail: phil.koenig@gmx.de
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.9
import QtQuick.Controls 1.5
import QtQuick.Layouts 1.3
import MuseScore 3.0

MuseScore {
      version: "3.3"
      menuPath: "Plugins.FlötenGriffe"
      description: "Schreibt Griffe der Spielmannsflöte auswählbar für die Stimmungen in Bb und Eb unter die Noten"
      pluginType: "dialog"

// ------ OPTIONS -------

    id: window
    width:360
    height: 180
    RowLayout {
        id: column
        anchors.margins : 30
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: 10
        x: 150
    Label {
      font.pointSize: 14
      text: "Stimmung auswählen"  //  <I> effect     ,   <B> <H1> no effect
    }
        ComboBox {
            currentIndex: 0
            implicitWidth: 155
            model: ListModel {
                id: keylist
                property var key
                ListElement { text: "Bb (Sopran/Lyra)"; tabkey: 2 }
                ListElement { text: "Eb (Alt)"; tabkey: -3 }
            }
            width: 100
            onCurrentIndexChanged: {
                console.debug(keylist.get(currentIndex).text + ", " + keylist.get(currentIndex).tabkey)
                keylist.key = keylist.get(currentIndex).tabkey
            }
        }
    }
    RowLayout {
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: column.bottom
        height: 150
        Button {
            id: okButton
            text: "Ok"
            onClicked: {
                apply()
                Qt.quit()
            }
        }
        Button {
            id: closeButton
            text: "Schließen"
            onClicked: { Qt.quit() }
        }

    }
      
      function getNoteName (tpc) {
            var noteName = "";
            switch (tpc) {
                  case 62: return "1\r\n/\r\n7"; break;  		       //d
                  case 63: return "1\r\n/\r\n6"; break;  		       //dis/es
                  case 64: return "1\r\n/\r\n5"; break;  		       //e
                  case 65: return "1\r\n2\r\n3\r\n4\r\n6\r\n7"; break;           //f
                  case 66: return "1\r\n2\r\n3\r\n5\r\n6\r\n7"; break;           //fis/ges
                  case 67: return "1\r\n2\r\n3"; break;  		      //g
                  case 68: return "1\r\n2\r\n4\r\n5\r\n6\r\n7"; break;          //gis/as
                  case 69: return "1\r\n2"; break;  	                          //a
                  case 70: return "1\r\n3\r\n4\r\n5\r\n6"; break;            	//ais/b
                  case 71: return "1"; break;  				//h
                  case 72: return "2\r\n3"; break;  			//c
                  case 73: return "4\r\n5\r\n6"; break;  		//cis/des	
                  case 74: return "2\r\n/\r\n7"; break;  		//d
                  case 75: return "2\r\n/\r\n6"; break;  		//dis/es
                  case 76: return "1\r\n/\r\n5"; break;  		//e
                  case 77: return "1\r\n2\r\n3\r\n4\r\n6"; break; 	//f
                  case 78: return "1\r\n2\r\n3\r\n5\r\n6"; break;  	//fis/ges
                  case 79: return "1\r\n2\r\n3"; break;  		//g
                  case 80: return "1\r\n2\r\n4"; break;  		//gis/as
                  case 81: return "1\r\n2"; break;  		//a
                  case 82: return "1\r\n3"; break;  		//ais/b
                  case 83: return "1"; break;  				//h
                  case 84: return "1\r\n3\r\n4\r\n6"; break;  		//c
                  case 85: return "4\r\n5\r\n6"; break;  		//cis/des	
                  case 86: return "2\r\n/\r\n3"; break;  		//d
                  case 87: return "2\r\n3\r\n5\r\n6"; break;  		//dis/es
                  case 88: return "1\r\n2\r\n5\r\n6"; break;  		//e
                  case 89: return "1\r\n2\r\n4"; break;  		//f
                  case 90: return "1\r\n2\r\n4\r\n5\r\n6\r\n7"; break; //fis/ges
                  case 91: return "1\r\n3\r\n7"; break;  		//g
                  case 92: return "3\r\n6"; break;  			//gis/as
                  case 93: return "2\r\n/\r\n7"; break;  		//a	
            } 
            return noteName;
      }
      
      function getFingering(notes) {
            var tabkey = keylist.key
            console.log("tabkey set to  " + keylist.key)
            var text = "";
            for (var i = 0; i < notes.length; i++) {
                  var index = notes[i].pitch + tabkey;
                  console.log("Pitch: " + index + " | NoteName: " + getNoteName(index));
                  text = getNoteName(index);
            }
            return text;
      }
      
    function applyToSelection() {
            if (typeof curScore === 'undefined')
                Qt.quit();
            console.log("Start of program")
            var seperator =  "\n";
            var xPos = 1;
            var yPos =  100;
            var offsetFactor = 20.3;
            var textposition = Placement.BELOW;

            var fontSize = 18;
            
            var cursor = curScore.newCursor();
            var startStaff;
            var endStaff;
            var endTick;
            var fullScore = false;
            cursor.rewind(1);  // rewind to start of selection
            if (!cursor.segment) { // no selection
                  fullScore = true;
                  startStaff = 0; // start with 1st staff
                  endStaff  = curScore.nstaves - 1; // and end with last
            } 
            else {
                  startStaff = cursor.staffIdx;
                  cursor.rewind(2); // rewind to end of selection
                  if (cursor.tick == 0) {
                        endTick = curScore.lastSegment.tick + 1;
                  } 
                  else {
                        endTick = cursor.tick;
                  }
                  endStaff   = cursor.staffIdx;
            }
            for (var staff = startStaff; staff <= endStaff; staff++) {
                  cursor.rewind(1)
                  cursor.voice = 0;
                  cursor.staffIdx = staff;
                  
                  if (fullScore)  // no selection
                        cursor.rewind(0); // beginning of score
                  while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                        if (!cursor.element || cursor.element.type != Element.CHORD || (cursor.element.notes && cursor.element.notes.length > 0 && cursor.element.notes[0].tieBack)) {
                              cursor.next();
                              continue;
                        }

                        var fingering = getFingering(cursor.element.notes);
	   if (fingering) {
                              var text = newElement(Element.STAFF_TEXT);
                              var fontSizeTag="<font size=\""+fontSize+"\"/>";
                              text.text = fontSizeTag+fingering;
                              
                              text.offsetX= xPos;
                              text.offsetY = (cursor.element.notes.length - 1)*offsetFactor;//yPos;
                              text.placement = textposition;     
                              text.autoplace = false;                         

                              cursor.add(text);
                        }
                        cursor.next();
                  }
            }
            console.log("End of program")
            Qt.quit()
            }

    function apply() {
        curScore.startCmd()
        applyToSelection()
        curScore.endCmd()
    }

    onRun: {
        if (typeof curScore === 'undefined')
            Qt.quit();
    }
}
